@@ -207,7 +207,10 @@ impl Shell {
 
     /// Prints a red 'error' message.
     pub fn error<T: fmt::Display>(&mut self, message: T) -> CargoResult<()> {
-        self.print(&"error:", Some(&message), Red, false)
+        if self.needs_clear {
+            self.err_erase_line();
+        }
+        self.err.print(&"error:", Some(&message), Red, false)
     }
 
     /// Prints an amber 'warning' message.
@@ -363,7 +366,7 @@ impl ColorChoice {
     }
 }
 
-#[cfg(any(target_os = "linux", target_os = "macos"))]
+#[cfg(any(target_os = "linux", target_os = "macos", target_os = "freebsd"))]
 mod imp {
     use std::mem;
 
@@ -374,7 +377,7 @@ mod imp {
     pub fn stderr_width() -> Option<usize> {
         unsafe {
             let mut winsize: libc::winsize = mem::zeroed();
-            if libc::ioctl(libc::STDERR_FILENO, libc::TIOCGWINSZ, &mut winsize) < 0 {
+            if libc::ioctl(libc::STDERR_FILENO, libc::TIOCGWINSZ.into(), &mut winsize) < 0 {
                 return None;
             }
             if winsize.ws_col > 0 {
@@ -393,7 +396,10 @@ mod imp {
     }
 }
 
-#[cfg(all(unix, not(any(target_os = "linux", target_os = "macos"))))]
+#[cfg(all(
+    unix,
+    not(any(target_os = "linux", target_os = "macos", target_os = "freebsd"))
+))]
 mod imp {
     pub(super) use super::default_err_erase_line as err_erase_line;
 
@@ -458,7 +464,13 @@ mod imp {
     }
 }
 
-#[cfg(any(all(unix, not(any(target_os = "linux", target_os = "macos"))), windows,))]
+#[cfg(any(
+    all(
+        unix,
+        not(any(target_os = "linux", target_os = "macos", target_os = "freebsd"))
+    ),
+    windows,
+))]
 fn default_err_erase_line(shell: &mut Shell) {
     if let Some(max_width) = imp::stderr_width() {
         let blank = " ".repeat(max_width);
