@@ -1,10 +1,10 @@
 use std::fmt;
 use std::str::FromStr;
 
+use crate::support::registry::Package;
+use crate::support::rustc_host;
+use crate::support::{basic_manifest, project};
 use cargo::util::{Cfg, CfgExpr};
-use support::registry::Package;
-use support::rustc_host;
-use support::{basic_manifest, project};
 
 macro_rules! c {
     ($a:ident) => {
@@ -54,7 +54,7 @@ where
     );
 }
 
-#[test]
+#[cargo_test]
 fn cfg_syntax() {
     good("foo", c!(foo));
     good("_bar", c!(_bar));
@@ -66,7 +66,7 @@ fn cfg_syntax() {
     good("foo = \"3 e\"", c!(foo = "3 e"));
 }
 
-#[test]
+#[cargo_test]
 fn cfg_syntax_bad() {
     bad::<Cfg>("", "found nothing");
     bad::<Cfg>(" ", "found nothing");
@@ -81,7 +81,7 @@ fn cfg_syntax_bad() {
     bad::<Cfg>("foo, bar", "malformed cfg value");
 }
 
-#[test]
+#[cargo_test]
 fn cfg_expr() {
     good("foo", e!(foo));
     good("_bar", e!(_bar));
@@ -100,7 +100,7 @@ fn cfg_expr() {
     good("not(all(a))", e!(not(all(a))));
 }
 
-#[test]
+#[cargo_test]
 fn cfg_expr_bad() {
     bad::<CfgExpr>(" ", "found nothing");
     bad::<CfgExpr>(" all", "expected `(`");
@@ -112,7 +112,7 @@ fn cfg_expr_bad() {
     bad::<CfgExpr>("foo(a)", "consider using all() or any() explicitly");
 }
 
-#[test]
+#[cargo_test]
 fn cfg_matches() {
     assert!(e!(foo).matches(&[c!(bar), c!(foo), c!(baz)]));
     assert!(e!(any(foo)).matches(&[c!(bar), c!(foo), c!(baz)]));
@@ -140,7 +140,7 @@ fn cfg_matches() {
     assert!(!e!(any((not(foo)), (all(foo, bar)))).matches(&[c!(foo)]));
 }
 
-#[test]
+#[cargo_test]
 fn cfg_easy() {
     let p = project()
         .file(
@@ -156,14 +156,15 @@ fn cfg_easy() {
             [target."cfg(windows)".dependencies]
             b = { path = 'b' }
         "#,
-        ).file("src/lib.rs", "extern crate b;")
+        )
+        .file("src/lib.rs", "extern crate b;")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "")
         .build();
     p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn dont_include() {
     let other_family = if cfg!(unix) { "windows" } else { "unix" };
     let p = project()
@@ -181,7 +182,8 @@ fn dont_include() {
         "#,
                 other_family
             ),
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "")
         .build();
@@ -191,10 +193,11 @@ fn dont_include() {
 [COMPILING] a v0.0.1 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn works_through_the_registry() {
     Package::new("baz", "0.1.0").publish();
     Package::new("bar", "0.1.0")
@@ -214,10 +217,12 @@ fn works_through_the_registry() {
             [dependencies]
             bar = "0.1.0"
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             "#[allow(unused_extern_crates)] extern crate bar;",
-        ).build();
+        )
+        .build();
 
     p.cargo("build")
         .with_stderr(
@@ -231,10 +236,11 @@ fn works_through_the_registry() {
 [COMPILING] foo v0.0.1 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn ignore_version_from_other_platform() {
     let this_family = if cfg!(unix) { "unix" } else { "windows" };
     let other_family = if cfg!(unix) { "windows" } else { "unix" };
@@ -259,10 +265,12 @@ fn ignore_version_from_other_platform() {
         "#,
                 this_family, other_family
             ),
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             "#[allow(unused_extern_crates)] extern crate bar;",
-        ).build();
+        )
+        .build();
 
     p.cargo("build")
         .with_stderr(
@@ -274,10 +282,11 @@ fn ignore_version_from_other_platform() {
 [COMPILING] foo v0.0.1 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_target_spec() {
     let p = project()
         .file(
@@ -291,7 +300,8 @@ fn bad_target_spec() {
             [target.'cfg(4)'.dependencies]
             bar = "0.1.0"
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
 
     p.cargo("build")
@@ -306,10 +316,11 @@ Caused by:
 Caused by:
   unexpected character in cfg `4`, [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_target_spec2() {
     let p = project()
         .file(
@@ -323,7 +334,8 @@ fn bad_target_spec2() {
             [target.'cfg(bar =)'.dependencies]
             baz = "0.1.0"
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
 
     p.cargo("build")
@@ -338,10 +350,11 @@ Caused by:
 Caused by:
   expected a string, found nothing
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn multiple_match_ok() {
     let p = project()
         .file(
@@ -369,14 +382,15 @@ fn multiple_match_ok() {
         "#,
                 rustc_host()
             ),
-        ).file("src/lib.rs", "extern crate b;")
+        )
+        .file("src/lib.rs", "extern crate b;")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "")
         .build();
     p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn any_ok() {
     let p = project()
         .file(
@@ -390,7 +404,8 @@ fn any_ok() {
             [target."cfg(any(windows, unix))".dependencies]
             b = { path = 'b' }
         "#,
-        ).file("src/lib.rs", "extern crate b;")
+        )
+        .file("src/lib.rs", "extern crate b;")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "")
         .build();
@@ -398,12 +413,8 @@ fn any_ok() {
 }
 
 // https://github.com/rust-lang/cargo/issues/5313
-#[test]
-#[cfg(all(
-    target_arch = "x86_64",
-    target_os = "linux",
-    target_env = "gnu"
-))]
+#[cargo_test]
+#[cfg(all(target_arch = "x86_64", target_os = "linux", target_env = "gnu"))]
 fn cfg_looks_at_rustflags_for_target() {
     let p = project()
         .file(
@@ -417,7 +428,8 @@ fn cfg_looks_at_rustflags_for_target() {
             [target.'cfg(with_b)'.dependencies]
             b = { path = 'b' }
         "#,
-        ).file(
+        )
+        .file(
             "src/main.rs",
             r#"
             #[cfg(with_b)]
@@ -425,7 +437,8 @@ fn cfg_looks_at_rustflags_for_target() {
 
             fn main() { b::foo(); }
         "#,
-        ).file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
+        )
+        .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "pub fn foo() {}")
         .build();
 
